function [nout] = cohist(x,y,m,d)
% Co-occurance matrix or joint PDF.
% N = COHIST(X,Y) correlates the elements of X and Y into 32x32 equally 
% spaced containers and returns the number of elements in each container.
% X and Y are matrices of type double with range [0,1] with the same
% number of rows and columns.
%
% N = COHIST(Y,X,M), where M is a scalar, uses M bins.
%
% N = COHIST(Y,X,M,D), where D is a scalar, uses D decades of scaling.
%
% COHIST(...) without output arguments produces a histogram image of
% the results.
% R.M. Clodfelter, June 1998
% Copyright (c) 1998 by R.M. Clodfelter
% $Revision: 1.00 $ $Date: 1998/06/16 $

if nargin <= 1
    error('Requires two or three input arguments.')
end

if nargin == 2
   m = 32;
end

if nargin <= 3
    d = 0;
end

if isstr(x) | isstr(y) | isstr(m)
    error('Input arguments must be numeric.')
end

if size(x) ~= size(y)
    error('Input matrices must be the same size.');
end

if isa(x, 'uint8')
    x = double(x(:))/255;
elseif isa(x, 'double')
    x = x(:);
else
    error('Unsupported input class');
end

if isa(y, 'uint8')
    y = double(y(:))/255;
elseif isa(y, 'double')
    y = y(:);
else
    error('Unsupported input class');
end

if max(y) > 1 | max(x) > 1 | min(y) < 0 | min(x) < 0 
    error('Inputs out of range (0-255 uint8) or (0-1 double)');
end

nn = zeros(1,(m+1)^2);
x = floor(x.*m);
y = floor(y.*m);
n = sort(x*(m+1)+y);
z = find([n;-1]~=[-1;n]);
cnt = diff(z);
nn(n(z(1:(length(z)-1)))+1) = cnt;
nn = reshape(nn,m+1,m+1);
nn(:,m) = nn(:,m)+nn(:,m+1);
nn(m,:) = nn(m,:)+nn(m+1,:);

if nargout == 0
    nn = nn(1:m,1:m);
    if d ~= 0
        t = 10^-d;
        nn = nn.*(nn>=t) + (nn<t)*t;
        nn = 1.0+log10(nn)/d;
    end
    if m < 128
        t = 8 - ceil(log2(m));
        nn = interp2(nn,t);
    end
    [xmax ymax] = size(nn);
    mnn = max(nn(:));
    imshow((mnn-nn)/mnn);
    h = gcf;
    ha = get(h,'CurrentAxes');
    set(ha,'Color',[0 0 0]);
    set(ha,'Visible','On');
    set(ha,'YDir','normal');
    %set(ha,'YTickLabel',[]);
    %set(ha,'XTickLabel',[]);
    %set(ha,'YTick',[]);
    %set(ha,'XTick',[]);
    %set(ha,'XLim',[0 m]);
    %set(ha,'YLim',[0 m]);
    %get(ha)
    %get(ha,'CurrentPoint')
    %get(h)
else
    nout = nn;
end

